#!/usr/bin/env python3
import sentry_sdk
import time
import requests
import requests.exceptions
import os
import json
import logging
import socket

from redis_super_storage import RedisSuperStorage

"""
Scheduler
"""

__author__ = "@kocsisr"
__copyright__ = "Copyright 2020, GoldenPogácsa Team"
__module_name__ = "app"
__version__text__ = "1"

sentry_sdk.init("https://0a106e104e114bc9a3fa47f9cb0db2f4@sentry.kmlabz.com/10")



class Scheduler:

    def __init__(self):
        pass

    def request_first_sync(self, ip_list, consumer_list_redis):
        temp_dict = {}
        for ip in ip_list:
            try:
                # request synchronization
                response = requests.post(f"http://{ip}/sync", json={'uuid': os.environ['LOCAL_UUID']}, timeout=5)
            except (requests.exceptions.ConnectionError, requests.exceptions.Timeout) as e:
                logging.error(f"Error while syncing to {ip}: {str(e)}")
                continue

            if response.status_code == 200:
                temp_dict[response.json()['uuid']] = {'ip': ip}
        consumer_list_redis.update(temp_dict)
        self.r.set('consumer_list', json.dumps(consumer_list_redis).encode('utf-8'))
        logging.debug('Update redis consumers ip list from first answers: Done')

    def request_sync(self, consumer_list_redis):


    def test_ip_change(self, host_name):

        old_ip = self.r.get('current_ip')

        if old_ip:
            old_ip = old_ip.decode('utf-8')

        current_ip = socket.gethostbyname(host_name)

        if not old_ip:  # Not set yet. I this case no update required
            self.r.set('current_ip', current_ip.encode('utf-8'))
            logging.debug(f"Previous info about the ip address could not be found! Current: {current_ip}")

        elif old_ip != current_ip:
            logging.info(f'IP changed: {old_ip} -> {current_ip} Pushing updates...')

            # pushing updates...
            keys = self.r.keys('producer_*')

            logging.debug(f'Pushing update to the following producers: ' + ', '.join(k.decode('utf-8') for k in keys))

            for key in keys:
                ip = self.r.get(key)
                if ip:
                    ip = ip.decode('utf-8')
                else:
                    continue


        else:
            logging.debug(f'IP unchanged: {current_ip}')

        logging.debug('Waiting for next turn')
        time.sleep(os.environ.get("RUN_INTERVAL", 30))


def main():
    # set logging preferences
    logging.basicConfig(filename='', level=logging.DEBUG)

    redis_storage = RedisSuperStorage(os.environ.get('REDIS_URL', "redis://localhost:6379/0"))
    scheduler = Scheduler()



if __name__ == "__main__":
    try:

        main()
    except KeyboardInterrupt:
        pass
